# Copyright (c) 2008 Douglas Mayle
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from datetime import datetime, timedelta
from time import sleep

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

class Greylist(models.Model):
    """
    A simple greylist model useful for Django projects.
    
    This model should be instantiated with the get_or_create method to ensure unique entries.

    >>> entry = Greylist.entries.get_or_create(source='127.0.0.1', sender='Author', receiver='Destination')[0]
    >>> entry.save()
    >>> if not entry.verify():
    ...     raise forms.ValidationError('Commenting overloaded, please try again in 5 seconds.')
    """
    source = models.IPAddressField()
    sender = models.CharField(_(u'Sender'), max_length=settings.NAME_LENGTH)
    receiver = models.CharField(_(u'Receiver'), max_length=settings.NAME_LENGTH)
    created = models.DateTimeField(_(u'Created'), null=True)
    updated = models.DateTimeField(_(u'Updated'), null=True)

    class Meta:
        unique_together = ('source','sender','receiver')

    entries = models.Manager()

    def __unicode__(self):
        return u'From: %s - To: %s - By: %s' % (self.sender, self.receiver, self.source)

    def save(self, *args, **kwargs):
        now = datetime.now()
        if not self.created:
            self.created = now
        self.updated = now
        return super(Greylist, self).save(*args, **kwargs)

    def verify(self, delay=5, tarpit=True):
        """
        Method to check if the greylist entry exists.

        If 'tarpit' is true (the default), this method will pause 'delay' seconds before returning.
        """
        now = datetime.now()
        if not self.created or self.created > (now - timedelta(seconds=10)):
            if tarpit:
                sleep(delay)
            return False
        return True


